from enum import Enum
from time import time
from typing import Literal

from aiohttp import ClientSession, BaseConnector
from jwt import decode

from .types import *


class SaturnPlatform(Enum):
    SATURN_WEB = "web"


API_URL = "https://saturn.live/backend/prod/api/v2/"


class SaturnLiveClient:
    def __init__(
            self,
            access_token: str,
            refresh_token: str,
            *,
            user_agent: str | None = None,
            platform: SaturnPlatform = SaturnPlatform.SATURN_WEB,
            connector: BaseConnector | None = None
    ):
        self.platform: SaturnPlatform = platform
        self.session: ClientSession = ClientSession(connector=connector)

        self.refresh_token: str = refresh_token
        self.token: str = access_token
        self.user_agent: str | None = user_agent

        self.identity: Identity | None = None

    async def on_token_change(self, access_token: str) -> None:
        """For saving the token to a file or database so you can log in later."""
        pass

    async def authorize(self, access_token: str, refresh_token: str) -> tuple[str, Identity, str | None]:
        async with self.session.post(
                "https://saturn.live/refresh/prod/auth",
                json={"access_token": access_token, "refresh_token": refresh_token}
        ) as resp:
            if resp.status == 200:
                json_data: dict = await resp.json()
                return json_data["access_token"], Identity.from_dict(json_data["identity"]), json_data[
                    "refresh_token"] or refresh_token
            else:
                raise resp.raise_for_status()

    async def get_token(self) -> None:
        if self.token:
            decoded_token: dict = decode(self.token, options={'verify_signature': False}, algorithms=["HS256"])
            if decoded_token["exp"] > time():
                return
        self.token, self.identity, self.refresh_token = await self.authorize(self.refresh_token, self.refresh_token)
        await self.on_token_change(self.token)

    async def get_headers(self) -> dict:
        headers: dict = {
            "Content-Type": "application/json",
            "Accept": "application/json, text/plain, */*",
            "DNT": "1",
            "X-Saturn-Platform": self.platform.value,
        }
        if self.user_agent:
            headers["User-Agent"] = self.user_agent
        await self.get_token()
        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    async def get_student(self, student_id: int | Literal["me"]) -> Student:
        async with self.session.get(f"{API_URL}user/{student_id}", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                json_data: dict = await resp.json()
                if json_data["public"]:
                    return FullStudent.from_dict(json_data)
                else:
                    return Student.from_dict(json_data)  # Private accounts are missing data.
            else:
                raise resp.raise_for_status()

    async def get_calendar(self, school_id: str) -> list[CalendarDay]:
        async with self.session.get(f"{API_URL}schools/{school_id}/calendar", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [CalendarDay.from_dict(day) for day in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_all_courses(self, school_id: str) -> list[Course]:
        async with self.session.get(f"{API_URL}schools/{school_id}/courses", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [Course.from_dict(course) for course in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_courses(self, student_id: int) -> list[DefinedCourse]:
        async with self.session.get(f"{API_URL}user/{student_id}/courses", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [DefinedCourse.from_dict(course) for course in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_emojis(self) -> list[Emoji]:
        async with self.session.get(f"{API_URL}emojis", **await self.get_headers()) as resp:
            if resp.status == 200:
                return [Emoji.from_dict(emoji) for emoji in await resp.json()]
            else:
                raise resp.raise_for_status()

    # TODO: All chat endpoints

    # TODO: All sports endpoints (minus teams)

    # TODO: Sharing status endpoints

    async def get_schedules(self, student_id: int, *, include_chats: bool) -> list[BellSchedule]:
        async with self.session.get(f"{API_URL}user/{student_id}/schedule",
                                    params={"include_chats": str(include_chats)},
                                    headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [BellSchedule.from_dict(schedule) for schedule in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_schedule_changes(self, school_id: str) -> list[ScheduleChange]:
        async with self.session.get(f"{API_URL}network/reports/schedule_change", params={"school_id": school_id},
                                    headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [ScheduleChange.from_dict(change) for change in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_all_staff(self, school_id: str) -> list[Staff]:
        async with self.session.get(f"{API_URL}schools/{school_id}/staff", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [Staff.from_dict(staff) for staff in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_tasks(self) -> list[Task]:
        async with self.session.get(f"{API_URL}tasks", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [Task.from_dict(task) for task in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def add_task(self, task: Task) -> None:
        async with self.session.post(f"{API_URL}tasks", json=dict(task), headers=await self.get_headers()) as resp:
            if resp.status != 200:
                raise resp.raise_for_status()

    async def remove_task(self, task_id: int) -> None:
        async with self.session.delete(f"{API_URL}tasks/{task_id}", headers=await self.get_headers()) as resp:
            if resp.status != 204:
                raise resp.raise_for_status()

    async def get_teams(self, school_id: str) -> list[Team]:
        async with self.session.get(f"{API_URL}events/{school_id}/teams", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [Team.from_dict(team) for team in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def get_students(self, school_id: str) -> list[Student]:
        async with self.session.get(f"{API_URL}schools/{school_id}/users", headers=await self.get_headers()) as resp:
            if resp.status == 200:
                return [Student.from_dict(student) for student in await resp.json()]
            else:
                raise resp.raise_for_status()

    async def close(self):
        await self.session.close()


__all__ = ("SaturnLiveClient", "SaturnPlatform")



from datetime import datetime
from typing import Any
from uuid import UUID


class SizeUrls:
    __slots__ = ("large", "medium", "small")

    def __init__(self, *, large: str, medium: str, small: str):
        self.large: str = large
        self.medium: str = medium
        self.small: str = small

    @classmethod
    def from_dict(cls, d: dict):
        return cls(**d)

    def __dict__(self):
        return {
            "large": self.large,
            "medium": self.medium,
            "small": self.small,
        }


class Image:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = ("content_type", "id", "is_bitmoji", "metadata", "resource_url", "size_urls")

    def __init__(
            self,
            *,
            content_type: str,
            id: str,
            is_bitmoji: bool,
            metadata: dict[str, int] | None,
            resource_url: str,
            size_urls: SizeUrls
    ):
        self.content_type: str = content_type
        self.id: str = id
        self.is_bitmoji: bool = is_bitmoji
        self.metadata: dict[str, int] | None = metadata
        self.resource_url: str = resource_url
        self.size_urls: SizeUrls = size_urls

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["size_urls"] = SizeUrls.from_dict(d["size_urls"])
        d["metadata"] = d.get("metadata")
        d["is_bitmoji"] = d.get("is_bitmoji", False)
        return cls(**d)

    def __dict__(self):
        return {
            "content_type": self.content_type,
            "id": self.id,
            "is_bitmoji": self.is_bitmoji,
            "metadata": self.metadata,
            "resource_url": self.resource_url,
            "size_urls": dict(self.size_urls),
        }


class BaseStudent:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = ("id", "name")

    def __init__(self, *, id: int, name: str):
        self.id: int = id
        self.name: str = name

    @classmethod
    def from_dict(cls, d: dict):
        return cls(**d)


class Student(BaseStudent):
    __slots__ = BaseStudent.__slots__ + (
        "ambassador_school",
        "bio",
        "birthday",
        "created_at",
        "email",
        "first_name",
        "grade",
        "hidden",
        "is_ambassador",
        "last_name",
        "profile_picture",
        "public",
        "url",
        "user_cohort",
        "user_instagram",
        "user_snapchat",
        "user_tiktok",
        "user_venmo",
        "user_vsco",
        "updated_at",
    )

    def __init__(
            self,
            *,
            ambassador_school: str | None,
            bio: str | None,
            birthday: datetime,
            created_at: datetime,
            email: str | None,
            first_name: str,
            grade: int,
            hidden: Any | None,  # Unknown
            id: int,
            is_ambassador: bool,
            last_name: str,
            name: str,
            profile_picture: Image | None,
            public: bool,
            url: str | None,
            user_cohort: str | None,
            user_instagram: str | None,
            user_snapchat: str | None,
            user_tiktok: str | None,
            user_venmo: str | None,
            user_vsco: str | None,
    ):
        super().__init__(id=id, name=name)
        self.ambassador_school: str | None = ambassador_school
        self.bio: str | None = bio
        self.birthday: datetime = birthday
        self.created_at: datetime = created_at
        self.email: str | None = email
        self.first_name: str = first_name
        self.grade: int = grade
        self.hidden: Any | None = hidden
        self.is_ambassador: bool = is_ambassador
        self.last_name: str = last_name
        self.profile_picture: Image | None = profile_picture
        self.public: bool = public
        self.url: str | None = url
        self.user_cohort: str | None = user_cohort
        self.user_instagram: str | None = user_instagram
        self.user_snapchat: str | None = user_snapchat
        self.user_tiktok: str | None = user_tiktok
        self.user_venmo: str | None = user_venmo
        self.user_vsco: str | None = user_vsco

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["created_at"] = datetime.fromisoformat(d["created_at"])
        if "profile_picture" in d and d["profile_picture"]:
            d["profile_picture"] = Image.from_dict(d["profile_picture"])
        else:
            d["profile_picture"] = None
        if d.get("birthday"):
            d["birthday"] = datetime.strptime(d["birthday"], "%Y-%m-%d")
        return cls(**d)


class Staff:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = ("id", "name", "suggested")

    def __init__(self, *, id: int, name: str, suggested: bool):
        self.id: int = id
        self.name: str = name
        self.suggested: bool = suggested

    @classmethod
    def from_dict(cls, d: dict):
        return cls(**d)


class Asset:
    __slots__ = ("gif", "png", "unicode", "unicode_code")

    def __init__(
            self,
            *,
            gif: Image | None,
            png: Image | None,
            unicode: str,
            unicode_code: str
    ):
        self.gif: Image | None = gif
        self.png: Image | None = png
        self.unicode: str = unicode
        self.unicode_code: str = unicode_code

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        if d.get("gif"):
            d["gif"] = Image.from_dict(d["gif"])
        if d.get("png"):
            d["png"] = Image.from_dict(d["png"])
        return cls(**d)


class Emoji:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.position == other.position
        )

    __slots__ = ("category", "name", "position", "resources", "schedule_snapchat_sticker", "tags", "snapchat_sticker")

    def __init__(
            self,
            *,
            category: str,
            name: str,
            position: int,
            resources: Asset,
            schedule_snapchat_sticker: str | None,
            snapchat_sticker: str | None,
            tags: list[str]
    ):
        self.category: str = category
        self.name: str = name
        self.position: int = position
        self.resources: Asset = resources
        self.schedule_snapchat_sticker: str | None = schedule_snapchat_sticker
        self.snapchat_sticker: str | None = snapchat_sticker
        self.tags: list[str] = tags

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["resources"] = Asset.from_dict(d["resources"])
        d["snapchat_sticker"] = d.get("snapchat_sticker") or d.get("schedule_snapchat_sticker")
        return cls(**d)


class Course:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = ("emoji", "id", "name", "school", "suggested")

    def __init__(
            self,
            *,
            emoji: Emoji,
            id: int,
            name: str,
            school: str,
            suggested: bool
    ):
        self.emoji: Emoji = emoji
        self.id: int = id
        self.name: str = name
        self.school: str = school
        self.suggested: bool = suggested

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["emoji"] = Emoji.from_dict(d["emoji"])
        return cls(**d)


class Team:
    __slots__ = ("emoji_id", "gender", "id", "level", "name", "sport", "subscribed")

    def __init__(
            self,
            *,
            emoji_id: str,
            gender: str,
            id: UUID,
            level: str,
            name: str,
            sport: str,
            subscribed: bool,
    ):
        self.emoji_id: str = emoji_id
        self.gender: str = gender
        self.id: UUID = id
        self.level: str = level
        self.name: str = name
        self.sport: str = sport
        self.subscribed: bool = subscribed

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["id"] = UUID(d["id"])
        return cls(**d)


# TODO: I don't use tasks, so I can't get any data from it.


# TODO: Sharing status.


# TODO: Messages


# TODO: Chats


class DefinedCourse:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = (
        "block",
        "class_chat",
        "classmates",
        "course",
        "id",
        "meeting_times",
        "nickname",
        "room",
        "school",
        "staff"
    )

    def __init__(
            self,
            *,
            block: Any | None,  # Unknown
            class_chat: UUID | None,
            classmates: list[Student],
            course: Course,
            id: int,
            meeting_times: dict[UUID, list[UUID]],
            nickname: str | None,
            room: str,
            school: str,
            staff: list[Staff],
    ):
        self.block: Any | None = block
        self.class_chat: UUID | None = class_chat
        self.classmates: list[Student] = classmates
        self.course: Course = course
        self.id: int = id
        self.meeting_times: dict[UUID, list[UUID]] = meeting_times
        self.nickname: str | None = nickname
        self.room: str = room
        self.school: str = school
        self.staff: list[Staff] = staff

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        if d.get("class_chat"):
            d["class_chat"] = UUID(d["class_chat"])
        d["classmates"] = [Student.from_dict(s) for s in d.get("classmates", [])]
        d["course"] = Course.from_dict(d["course"])
        d["meeting_times"] = {UUID(k): [UUID(m) for m in v] for k, v in d["meeting_times"].items()}
        d["staff"] = [Staff.from_dict(s) for s in d["staff"]]
        return cls(**d)


class Period:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = (
        "day_type_id",
        "end_time",
        "id",
        "instance",
        "name",
        "period_type_id",
        "start_time"
    )

    def __init__(
            self,
            *,
            day_type_id: UUID,
            end_time: datetime,
            id: UUID,
            instance: DefinedCourse,
            name: str,
            period_type_id: UUID,
            start_time: datetime,
    ):
        self.day_type_id: UUID = day_type_id
        self.end_time: datetime = end_time
        self.id: UUID = id
        self.instance: DefinedCourse = instance
        self.name: str = name
        self.period_type_id: UUID = period_type_id
        self.start_time: datetime = start_time

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["day_type_id"] = UUID(d["day_type_id"])
        d["period_type_id"] = UUID(d["period_type_id"])
        d["id"] = UUID(d["id"])
        d["instance"] = DefinedCourse.from_dict(d["instance"])
        if d.get("start_time"):
            d["start_time"] = datetime.strptime(d["start_time"], "%H:%M:%S")
        if d.get("end_time"):
            d["end_time"] = datetime.strptime(d["start_time"], "%H:%M:%S")
        return cls(**d)


class BaseSchedule:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.id == other.id
        )

    __slots__ = (
        "display_name",
        "id",
        "name",
        "special",
        "static",
    )

    def __init__(
            self,
            *,
            display_name: str,
            id: UUID,
            name: str,
            special: bool,
            static: bool,
    ):
        self.display_name: str = display_name
        self.id: UUID = id
        self.name: str = name
        self.special: bool = special
        self.static: bool = static

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["id"] = UUID(d["id"])
        return cls(**d)


class BellSchedule(BaseSchedule):
    def __init__(
            self,
            *,
            author: BaseStudent,
            author_id: int,
            created_at: datetime,
            display_name: str,
            draft: bool,
            emoji: Emoji | None,
            emoji_id: str | None,
            hidden: bool,
            id: UUID,
            lunch_slot: Any | None,  # Unknown
            lunch_waves: list,  # Unknown
            name: str,
            order: int,
            periods: list[Period],
            school_id: str,
            special: bool,
            static: bool,
            updated_at: datetime,
    ):
        super().__init__(
            display_name=display_name,
            id=id,
            name=name,
            special=special,
            static=static,
        )
        self.author: BaseStudent = author
        self.author_id: int = author_id
        self.created_at: datetime = created_at
        self.draft: bool = draft
        self.emoji: Emoji | None = emoji
        self.emoji_id: str | None = emoji_id
        self.hidden: bool = hidden
        self.lunch_slot: Any | None = lunch_slot
        self.lunch_waves: list = lunch_waves
        self.order: int = order
        self.periods: list[Period] = periods
        self.school_id: str = school_id
        self.updated_at: datetime = updated_at

    __slots__ = BaseSchedule.__slots__ + (
        "author",
        "author_id",
        "created_at",
        "draft",
        "emoji",
        "emoji_id",
        "hidden",
        "lunch_slot",
        "lunch_waves",
        "order",
        "periods",
        "school_id",
        "updated_at",
    )

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["author"] = BaseStudent.from_dict(d["author"])
        d["periods"] = [Period.from_dict(p) for p in d["periods"]]
        d["id"] = UUID(d["id"])
        if d.get("emoji"):
            d["emoji"] = Emoji.from_dict(d["emoji"])
        return cls(**d)


class CalendarDay:
    def __init__(
            self,
            *,
            date: datetime,
            is_canceled: bool,
            raw_block_name: str,
            schedule: BaseSchedule | None,
    ):
        self.date: datetime = date
        self.is_canceled: bool = is_canceled
        self.raw_block_name: str = raw_block_name
        self.schedule: BaseSchedule | None = schedule

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["date"] = datetime.strptime(d["date"], "%Y-%m-%d")
        if d.get("schedule"):
            d["schedule"] = BaseSchedule.from_dict(d["schedule"])
        return cls(**d)


class ScheduleChange:
    def __init__(
            self,
            *,
            count: int,
            desired_schedule: str,
            desired_schedule_id: UUID,
            original_schedule: str,
            original_schedule_id: UUID,
            user_id: int,
    ):
        self.count: int = count
        self.desired_schedule: str = desired_schedule
        self.desired_schedule_id: UUID = desired_schedule_id
        self.original_schedule: str = original_schedule
        self.original_schedule_id: UUID = original_schedule_id
        self.user_id: int = user_id

    __slots__ = (
        "count",
        "desired_schedule",
        "desired_schedule_id",
        "original_schedule",
        "original_schedule_id",
        "user_id",
    )

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["desired_schedule_id"] = UUID(d["desired_schedule_id"])
        d["original_schedule_id"] = UUID(d["original_schedule_id"])
        return cls(**d)


class ScheduleChangeReport:
    def __eq__(self, other):
        return (
                isinstance(other, self.__class__)
                and self.key == other.key
        )

    def __init__(
            self,
            *,
            ambassadors: list[BaseStudent],
            changes: list[ScheduleChange],
            district_school_count: int,
            district_schools: list[str],
            key: int,
            report_count: int,
            school_name: str,
            school_title: str,
            school_state: str,
            status: str,
            target_date: datetime,
            timezone: str,  # We can (possibly) do better.
            upcoming_days: int,
            user_count: int,
    ):
        self.ambassadors: list[BaseStudent] = ambassadors
        self.changes: list[ScheduleChange] = changes
        self.district_school_count: int = district_school_count
        self.district_schools: list[str] = district_schools
        self.key: int = key
        self.report_count: int = report_count
        self.school_name: str = school_name
        self.school_title: str = school_title
        self.school_state: str = school_state
        self.status: str = status
        self.target_date: datetime = target_date
        self.timezone: str = timezone
        self.upcoming_days: int = upcoming_days
        self.user_count: int = user_count

    __slots__ = (
        "ambassadors",
        "changes",
        "district_school_count",
        "district_schools",
        "key",
        "report_count",
        "school_name",
        "school_title",
        "school_state",
        "status",
        "target_date",
        "timezone",
        "upcoming_days",
        "user_count",
    )

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["target_date"] = datetime.strptime(d["target_date"], "%Y-%m-%d")
        d["ambassadors"] = [BaseStudent.from_dict(a) for a in d["ambassadors"]]
        d["changes"] = [ScheduleChange.from_dict(c) for c in d["changes"]]
        return cls(**d)


class FullStudent(Student):
    __slots__ = Student.__slots__ + (
        "updated_at",
        "profile_pic_url",
        "gender",
        "gender_preference",
        "onboarded",
        "phone_number",
        "phone_validated",
        "tags",
        "granted_scopes",
        "school_id",
        "referred_by",
        "ambassador_school_id",
        "hashid",
        "school_title",
        "school",
        "waitlist_school",
        "courses",
        "permissions",
    )

    def __init__(
            self,
            *,
            updated_at: datetime | None,
            profile_pic_url: str,
            gender: Any | None,  # Unknown
            gender_preference: Any | None,  # Unknown
            onboarded: bool,
            phone_number: str,
            phone_validated: bool,
            tags: list[Any],  # Unknown
            granted_scopes: list[Any],  # Unknown
            school_id: str,
            referred_by: Any | None,  # Unknown
            ambassador_school_id: Any | None,  # Unknown
            hashid: str,
            school_title: str,
            school: str,
            waitlist_school: Any | None,  # Unknown
            courses: list[DefinedCourse],
            permissions: dict[str, bool],
            **kwargs,
    ):
        super().__init__(**kwargs)
        self.updated_at: datetime = updated_at
        self.profile_pic_url: str = profile_pic_url
        self.gender: Any | None = gender
        self.gender_preference: Any | None = gender_preference
        self.onboarded: bool = onboarded
        self.phone_number: str = phone_number
        self.phone_validated: bool = phone_validated
        self.tags: list[Any] = tags
        self.granted_scopes: list[Any] = granted_scopes
        self.school_id: str = school_id
        self.referred_by: Any | None = referred_by
        self.ambassador_school_id: Any | None = ambassador_school_id
        self.hashid: str = hashid
        self.school_title: str = school_title
        self.school: str = school
        self.waitlist_school: Any | None = waitlist_school
        self.courses: list[DefinedCourse] = courses
        self.permissions: dict[str, bool] = permissions

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        if d.get("updated_at"):
            d["updated_at"] = datetime.fromisoformat(d["updated_at"])
        else:
            d["updated_at"] = None
        d["courses"] = [DefinedCourse.from_dict(c) for c in d["courses"]]
        return cls(**d)


class Task:
    __slots__ = (
        "added_by",
        "attachments",
        "course_id",
        "course_slot",
        "created_at",
        "date_completed",
        "description",
        "due_date",
        "due_datetime",
        "due_seconds",
        "id",
        "images",
        "is_completed",
        "priority",
        "public",
        "shared_with",
        "title",
        "updated_at",
    )

    def __init__(
            self,
            *,
            added_by: list[BaseStudent],  # Unknown
            attachments: list[Any],  # Unknown
            course_id: UUID | None,
            course_slot: bool,
            created_at: datetime,
            date_completed: datetime | None,
            description: str | None,
            due_date: datetime,
            due_datetime: datetime,
            due_seconds: int,
            id: int,
            images: list[Image],  # Unknown
            is_completed: bool,
            priority: int,
            public: bool,
            shared_with: list[BaseStudent],
            title: str,
            updated_at: datetime,
    ):
        self.added_by: list[BaseStudent] = added_by
        self.attachments: list[Any] = attachments
        self.course_id: UUID = course_id
        self.course_slot: bool = course_slot
        self.created_at: datetime = created_at
        self.date_completed: datetime = date_completed
        self.description: str = description
        self.due_date: datetime = due_date
        self.due_datetime: datetime = due_datetime
        self.due_seconds: int = due_seconds
        self.id: int = id
        self.images: list[Image] = images
        self.is_completed: bool = is_completed
        self.priority: int = priority
        self.public: bool = public
        self.shared_with: list[BaseStudent] = shared_with
        self.title: str = title
        self.updated_at: datetime = updated_at

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["added_by"] = [BaseStudent.from_dict(s) for s in d["added_by"]]
        d["course_id"] = UUID(d["course_id"]) if d["course_id"] else None
        d["due_date"] = datetime.strptime(d["due_date"], "%Y-%m-%d")
        d["due_datetime"] = datetime.fromisoformat(d["due_datetime"])
        d["images"] = [Image.from_dict(i) for i in d["images"]]
        d["shared_with"] = [BaseStudent.from_dict(s) for s in d["shared_with"]]
        d["updated_at"] = datetime.fromisoformat(d["updated_at"])
        d["date_completed"] = datetime.fromisoformat(d["date_completed"]) if d["date_completed"] else None
        d["created_at"] = datetime.fromisoformat(d["created_at"])
        return cls(**d)

    def __dict__(self) -> dict:
        return {
            "added_by": [dict(s) for s in self.added_by],
            "attachments": self.attachments,
            "course_id": str(self.course_id) if self.course_id else None,
            "course_slot": self.course_slot,
            "created_at": self.created_at.isoformat(),
            "date_completed": self.date_completed.isoformat() if self.date_completed else None,
            "description": self.description,
            "due_date": self.due_date.isoformat(),
            "due_datetime": self.due_datetime.isoformat(),
            "due_seconds": self.due_seconds,
            "id": self.id,
            "images": [dict(i) for i in self.images],
            "is_completed": self.is_completed,
            "priority": self.priority,
            "public": self.public,
            "shared_with": [dict(s) for s in self.shared_with],
            "title": self.title,
            "updated_at": self.updated_at.isoformat(),
        }


class Identity:
    __slots__ = ("first_name", "last_name", "onboarded", "profile_pic", "school_id", "scopes")

    def __init__(
            self,
            *,
            first_name: str,
            last_name: str,
            onboarded: bool,
            profile_pic: Image,
            school_id: str,
            scopes: str,
    ):
        self.first_name: str = first_name
        self.last_name: str = last_name
        self.onboarded: bool = onboarded
        self.profile_pic: Image = profile_pic
        self.school_id: str = school_id
        self.scopes: str = scopes

    @classmethod
    def from_dict(cls, d: dict):
        d = d.copy()
        d["profile_pic"] = Image.from_dict(d["profile_pic"])
        return cls(**d)


__all__ = (
    "SizeUrls",
    "Image",
    "BaseStudent",
    "Student",
    "Staff",
    "Asset",
    "Emoji",
    "Course",
    "Team",
    "DefinedCourse",
    "Period",
    "BaseSchedule",
    "BellSchedule",
    "CalendarDay",
    "ScheduleChange",
    "ScheduleChangeReport",
    "FullStudent",
    "Task",
    "Identity",
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['saturnscrape']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT[crypto]>=2.3.0,<3.0.0', 'aiohttp>=3.8.1,<4.0.0']

setup_kwargs = {
    'name': 'saturnscrape',
    'version': '0.1.3',
    'description': 'Scrapes information from https://saturn.live, a collaborate school scheduling system.',
    'long_description': '# saturnscrape\n\nScrapes information from https://saturn.live, a collaborate school scheduling system.\n\nCurrently, this Python package only aims to support the scheduling and members of a school.\n\n## Example\n\n```python\nfrom asyncio import run\nfrom typing import cast\n\nfrom saturnscrape import *\n\n\nasync def runner():\n    client = SaturnLiveClient("jwt", "refresh token")\n    print(cast(FullStudent, await client.get_student("me")).phone_number)\n    await client.close()\n    \n\nrun(runner())\n```\n',
    'author': 'Parker Wahle',
    'author_email': 'parkeredwardwahle2017@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/regulad/saturnscrape',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
